/*--------------------------------------------------------------------------*/
//	Test program for <Direct3D 7 Utility Library for LCC-Win32>
//
//	by : Ngan Lo' 2001
//
//	email : nganlo8@email.com
/*--------------------------------------------------------------------------*/

// Includes -------------------------------------------------------

#define WIN32_LEAN_AND_MEAN
#define INITGUID

#include <math.h>
#include "d3d7wrap.h"

// Prototypes -----------------------------------------------------

// Game logic modules


// Globals --------------------------------------------------------
int x, y, GAMESPEED=14;
BOOL  DEBUGINFO=TRUE;
D3DCOLOR color;
D3DVERTEX g_Vertices[4];
float ry=0.05;
LPDIRECTDRAWSURFACE7 texture=NULL;

// Game initialization module here  -----------------------

void SetIdentityMatrix( D3DMATRIX& m )
{
    m._12 = m._13 = m._14 = m._21 = m._23 = m._24 = 0.0f;
    m._31 = m._32 = m._34 = m._41 = m._42 = m._43 = 0.0f;
    m._11 = m._22 = m._33 = m._44 = 1.0f;
}

void MatrixRotateY(D3DMATRIX& m, float angle)
{
	float c = cos(D3D7_DEG2RAD(angle));
	float s = sin(D3D7_DEG2RAD(angle));

	m._11 = c;
	m._31 = -s;
	m._13 = s;
	m._33 = c;
}

void SetVertex(D3DVERTEX& v, const D3DVECTOR& v1, const D3DVECTOR& vn, float tu, float tv)
{
	v.x=v1.x;
	v.y=v1.y;
	v.z=v1.z;
	v.nx=vn.x;
	v.ny=vn.y;
	v.nz=vn.z;
	v.tu=tu;
	v.tv=tv;
}

int Game_Init(void)
{
	// Initialize Direct3D
	D3DInit();

	color=RGB_MAKE(64,128,128);
	D3DClear(color);

	D3DVECTOR v, vn;

	// Create a polygon(quad) here, assign the texture uv too
	vn.x=vn.y=0.0f;
	vn.z=1.0f;
	v.z=0.0f;
	v.x=-2.0f; v.y=2.0f;
	SetVertex(g_Vertices[0],v,vn,0,0);
	v.x=2.0f; v.y=2.0f;
	SetVertex(g_Vertices[1],v,vn,1,0);
	v.x=-2.0f; v.y=-2.0f;
	SetVertex(g_Vertices[2],v,vn,0,1);
	v.x=2.0f; v.y=-2.0f;
	SetVertex(g_Vertices[3],v,vn,1,1);

	// Load the texture surface
	texture=D3DLoadTexture("santafe");

	// we need a Material for lighting, set the color to white
	D3DMATERIAL7 mtrl;
	ZeroMemory( &mtrl, sizeof(mtrl) );
	mtrl.ambient.r = 1.0f;
	mtrl.ambient.g = 1.0f;
	mtrl.ambient.b = 1.0f;
	IDirect3DDevice7_SetMaterial( g_pd3dDevice, &mtrl );
	IDirect3DDevice7_SetRenderState( g_pd3dDevice, D3DRENDERSTATE_AMBIENT, 0xffffffff );

	// Finally, setup World, View & Projection
	D3DMATRIX mat;
	SetIdentityMatrix(mat);
	IDirect3DDevice7_SetTransform( g_pd3dDevice, D3DTRANSFORMSTATE_WORLD, &mat );
	SetIdentityMatrix(mat);
	mat._43 = 10.0f;
    IDirect3DDevice7_SetTransform( g_pd3dDevice, D3DTRANSFORMSTATE_VIEW, &mat );
	SetIdentityMatrix(mat);
	mat._11 =  2.0f;
	mat._22 =  3.0f;
	mat._34 =  1.0f;
	mat._43 = -2.0f;
	mat._44 =  0.0f;
    IDirect3DDevice7_SetTransform( g_pd3dDevice, D3DTRANSFORMSTATE_PROJECTION, &mat );

	// Turn off the cull mode, so we can see the backface
	IDirect3DDevice7_SetRenderState( g_pd3dDevice, D3DRENDERSTATE_CULLMODE, D3DCULL_NONE );
	// Enable the Bilinear filtering
	IDirect3DDevice7_SetTextureStageState(g_pd3dDevice, 0, D3DTSS_MINFILTER, D3DTFN_LINEAR );
	IDirect3DDevice7_SetTextureStageState(g_pd3dDevice, 0, D3DTSS_MAGFILTER, D3DTFG_LINEAR );

	return (1);
}

// Game Shutdown module here ------------------------------

int Game_Shutdown(void)
{
	// remember to release the texture surface b4 quit
	if (texture!=NULL)
	{
		IDirectDrawSurface7_Release(texture);
		texture=NULL;
	}

	D3DShutdown();
	return (1);
}

// Game Main-Loop module here -----------------------------

int Game_Main(void)
{
	// Matrix use to polygon rotation
	D3DMATRIX mat;
	SetIdentityMatrix(mat);

	D3DClear(color);

	D3DBeginScene();

	// increase the rotation, in degree
	ry+=0.1;
	if (ry>360.0f) ry=0.0f;
	MatrixRotateY(mat, ry);
	IDirect3DDevice7_SetTransform( g_pd3dDevice, D3DTRANSFORMSTATE_WORLD, &mat );

	// set device to use the current texture
	IDirect3DDevice7_SetTexture(g_pd3dDevice,0,texture);

	// Draw the polygon
	IDirect3DDevice7_DrawPrimitive( g_pd3dDevice, D3DPT_TRIANGLESTRIP, D3DFVF_VERTEX, &g_Vertices, 4, 0L );

	// set the texture stage to NULL b4 end scene
	IDirect3DDevice7_SetTexture(g_pd3dDevice,0,NULL);

	D3DEndScene();

	D3DFlipWin();

	if (KEY_DOWN(VK_ESCAPE))
		PostMessage(g_hWnd,WM_DESTROY,0,0);

	return (1);
}

// Other game logic modules start here ----------------------------

