//----------------------------------------------------------------------------
// Direct3D 7 Wrapper Utility Library for Lccwin32
// 
// Created by : Ngan Lo' 2001 (nganlo8@email.com)
// 
// Free for personal use, if you want to use it in commercial purpose,
// please inform me 1st and credit my name in your programs.
//----------------------------------------------------------------------------

#ifndef _D3D7WRAP_H_
#define _D3D7WRAP_H_

#define WIN32_LEAN_AND_MEAN
#define STRICT

#include <windows.h>
#include <stdio.h>
#include <time.h>
#include <math.h>
#include <objbase.h>
#include <initguid.h>

#include <ddraw.h>
#include <d3d.h>

// defines for windows
#define WINDOW_CLASS_NAME "D3D7WRAPCLASS"  // WinClass name
#define WINDOWS_TITLE	"Ngan Lo's D3D7 sample" // WinClass title

#define WINDOW_WIDTH	640
#define WINDOW_HEIGHT	480
#define SCREEN_WIDTH	640
#define SCREEN_HEIGHT	480
#define BPP				16

#define D3DTEXTR_32BITSPERPIXEL   0x00000004
#define D3DTEXTR_16BITSPERPIXEL   0x00000008

#define D3D_PI ((FLOAT)3.14159265358979323846f)
#define D3D7_DEG2RAD(x) ((FLOAT)x*0.0174532925199432957692f)

#define KEY_DOWN(vk_code) ((GetAsyncKeyState(vk_code) & 0x8000) ? 1 : 0)
#define KEY_UP(vk_code)   ((GetAsyncKeyState(vk_code) & 0x8000) ? 0 : 1)
#define D3DRelease(x) { if(x) { x->lpVtbl->Release(x); x=NULL;} }

// Global structures
typedef struct
{
    DWORD dwDesiredBPP;   // Input for texture format search
    BOOL  bUseAlpha;
    BOOL  bUsePalette;
    BOOL  bFoundGoodFormat;

    DDPIXELFORMAT* pddpf; // Output of texture format search
}TEXTURESEARCHINFO;


// Global variables
LPDIRECTDRAW7			g_pDD;			// The DirectDraw object
LPDIRECT3D7				g_pD3D;
LPDIRECT3DDEVICE7		g_pd3dDevice;
LPDIRECTDRAWCLIPPER		g_pClipper;
LPDIRECTDRAWSURFACE7	g_pDDS;			// Primary surface
LPDIRECTDRAWSURFACE7	g_pDDSBack;		// Working Back surface
DDSURFACEDESC2			g_DDSD;
DDSCAPS2				g_DDScaps;
D3DDEVICEDESC7			g_D3DDD;
D3DVIEWPORT7			g_Viewport;
DDPIXELFORMAT			g_PixelFormat;

HWND					g_hWnd;			// To store the main windows handle
HINSTANCE				g_Hinst;		// To store the main instance
BOOL					d3dhardware;
int						g_iBpp;

RECT					g_rcWindow;		// Saves the window size & pos.
RECT					g_rcViewport;	// Pos. & size to blt from
RECT					g_rcScreen;		// Screen pos. for blt
BOOL					g_bActive;		// Game is running/active
BOOL					g_bReady;		// Game is ready for updates
BOOL					g_bWindowed=TRUE;	// Game is in windowed mode


//-- Window's Main functions
LRESULT CALLBACK WindowProc(HWND hwnd, UINT msg, WPARAM wparam, LPARAM lparam);
int WINAPI WinMain(	HINSTANCE hinstance, HINSTANCE hprevinstance, LPSTR lpcmdline, int ncmdshow);

//-- Game main modules
int Game_Init(void);
int Game_Shutdown(void);
int Game_Main(void);

//-- Wrapper function modules
void D3DFailed( LPCSTR s, BOOL QUITGAME );
BOOL D3DInit(void);
void D3DShutdown(void);
void D3DBeginScene(void);
void D3DEndScene(void);
BOOL D3DFlipWin(void);
BOOL D3DClear( D3DCOLOR color );


/*--------------------------------------------------------------------------*/
// CALLBACK  WindowProc !!!
LRESULT CALLBACK WindowProc(HWND hwnd, UINT msg, WPARAM wparam, LPARAM lparam)
{
	PAINTSTRUCT	ps;
	HDC			hdc;

	switch(msg)
	{
	case WM_CREATE:
        {
		// do initialization stuff here but NOT DirectX
		return(0);
		} break;

	case WM_ACTIVATE:
		{
            // Pause if minimized
            g_bActive = !((BOOL)HIWORD(wparam));
            return (0);
		} break;
    case WM_PAINT:
         {
         	// start painting
			hdc = BeginPaint(hwnd,&ps);
			ValidateRect( hwnd, NULL );
			
         	// end painting
			EndPaint(hwnd,&ps);
			return(0);
        } break;

	case WM_DESTROY:
		{
		// kill the application
		Game_Shutdown();
		PostQuitMessage(0);
		return(0);
		} break;
	case WM_SETCURSOR:
		{
			if (g_bActive && !g_bWindowed)
			{
				SetCursor(NULL);
				return TRUE;
			}
		} break;
	case WM_SYSCOMMAND:
		{
			// Prevent moving/sizing and power loss in fullscreen mode
			switch( wparam )
			{
				case SC_MOVE:
				case SC_SIZE:
				case SC_MAXIMIZE:
				case SC_KEYMENU:
				case SC_MONITORPOWER:
				if( !g_bWindowed )
					return 1;
					break;
			}
		} break;
	case WM_SIZE:
		{
		// Check if we are resizing our window...
		if (SIZE_MAXHIDE==wparam || SIZE_MINIMIZED==wparam)
			g_bActive = FALSE;
		else
			g_bActive = TRUE;
		} break;
	case WM_KEYDOWN:
		switch(wparam)
		{
			// Emergency Exit
			case VK_F12:
				Game_Shutdown();
				PostMessage(hwnd,WM_DESTROY,0,0);
			break;
		} break;

	default:break;
    }
	return (DefWindowProc(hwnd, msg, wparam, lparam));
}

/*--------------------------------------------------------------------------*/
// WinMain 
int WINAPI WinMain(	HINSTANCE hinstance, HINSTANCE hprevinstance,
					LPSTR lpcmdline, int ncmdshow)
{
	WNDCLASS winclass;
	HWND	 hwnd;
	MSG		 msg;
	RECT rc;

	// Initialize COM
	CoInitialize(NULL);
	
	winclass.style			= CS_HREDRAW | CS_VREDRAW;
	winclass.lpfnWndProc	= WindowProc;
	winclass.cbClsExtra		= 0;
	winclass.cbWndExtra		= 0;
	winclass.hInstance		= hinstance;
	winclass.hIcon			= LoadIcon(hinstance, MAKEINTRESOURCE(IDI_APPLICATION));
	winclass.hCursor		= LoadCursor(NULL, IDC_ARROW);
	winclass.hbrBackground	= GetStockObject(BLACK_BRUSH);
	winclass.lpszMenuName	= NULL;
	winclass.lpszClassName	= WINDOW_CLASS_NAME;
	
	// register the window class
	if (!RegisterClass(&winclass))
		return(0);

	// Calculate size of window based on desired client window size
	rc.left = 0;
	rc.top = 0;
	rc.right = WINDOW_WIDTH;
	rc.bottom = WINDOW_HEIGHT;
	//AdjustWindowRect( &rc, WS_OVERLAPPEDWINDOW, FALSE );
		
	hwnd = CreateWindow(WINDOW_CLASS_NAME,
					WINDOWS_TITLE,
					WS_OVERLAPPED|WS_SYSMENU,
					CW_USEDEFAULT, CW_USEDEFAULT,
					rc.right-rc.left, rc.bottom-rc.top,
					NULL, NULL, hinstance, NULL);
	
	// calculate the new window rect
	SIZE szDesiredClient;

	szDesiredClient.cx = WINDOW_WIDTH;
	szDesiredClient.cy = WINDOW_HEIGHT;

	//RECT rcNewWindowRect = CalculateWindowRect(hwnd, szDesiredClient);

	// resize the window according to the new rect
	//MoveWindow(hwnd, rcNewWindowRect.left, rcNewWindowRect.top, 
	//	(rcNewWindowRect.right - rcNewWindowRect.left),
	//	(rcNewWindowRect.bottom - rcNewWindowRect.top), TRUE);

	ShowWindow(hwnd,ncmdshow);
	UpdateWindow(hwnd);

	// Save the global hWnd & hInst values
	g_hWnd = hwnd;
	g_Hinst = hinstance;
	SetFocus( g_hWnd );
	
	InvalidateRect( g_hWnd, NULL, TRUE );
	
	Game_Init();

	// Enter main event loop
	while(1)
	{
		if (PeekMessage(&msg,NULL,0,0,PM_REMOVE))
		{
			if (msg.message == WM_QUIT)
				break;
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}
		else if (g_bActive)
        {
			Game_Main();
		}
		else WaitMessage();
	}
	Game_Shutdown();
	CoUninitialize();

	return(msg.wParam);
}

/*--------------------------------------------------------------------------*/
// D3DFailed : Handle error message dialog box !! Coooool !!
void D3DFailed(LPCSTR s, BOOL QUITGAME)
{
	char	szBuff[128];
	va_list vl;

    va_start(vl, s);
    vsprintf(szBuff, s, vl);
	MessageBox(g_hWnd,s,"Error",MB_OK);
	if (QUITGAME)
	{
		D3DShutdown();
   		PostQuitMessage(0);
	}
}

/*--------------------------------------------------------------------------*/
// D3DInit : Direct3D intialization
BOOL D3DInit(void)
{
	// Assume every PC have 3D VGA card, it's hard to find a PC doesn't have.
	d3dhardware=TRUE;
	
	// Create a DirectDraw Object 1st
	DirectDrawCreateEx( NULL, (VOID **)&g_pDD, &IID_IDirectDraw7, NULL );
	// We set it to windowed mode, well, easy to debug.
	IDirectDraw7_SetCooperativeLevel(g_pDD,g_hWnd, DDSCL_NORMAL);
	
	// Create Primary surface
	ZeroMemory( &g_DDSD, sizeof(DDSURFACEDESC2) );
	g_DDSD.dwSize = sizeof(g_DDSD);
	g_DDSD.dwFlags = DDSD_CAPS;
	g_DDSD.ddsCaps.dwCaps = DDSCAPS_PRIMARYSURFACE;
	IDirectDraw7_CreateSurface(g_pDD, &g_DDSD, &g_pDDS, NULL);
	
	// then create Clipper Object & attach to Primary surface
	DirectDrawCreateClipper(0, &g_pClipper, NULL);
	IDirectDrawClipper_SetHWnd(g_pClipper, 0, g_hWnd);
	IDirectDrawSurface7_SetClipper(g_pDDS,g_pClipper);
	
	// Create the Back Buffer & attach to Primary surface
	g_DDSD.dwFlags = DDSD_CAPS|DDSD_WIDTH|DDSD_HEIGHT;
	g_DDSD.dwWidth = SCREEN_WIDTH;
	g_DDSD.dwHeight = SCREEN_HEIGHT;
	g_DDSD.ddsCaps.dwCaps = DDSCAPS_OFFSCREENPLAIN | DDSCAPS_3DDEVICE;
	IDirectDraw7_CreateSurface(g_pDD, &g_DDSD, &g_pDDSBack, NULL);

	// Query the Direct3D interface to DirectDraw
	IDirectDraw7_QueryInterface(g_pDD, &IID_IDirect3D7, (VOID**)&g_pD3D );
	// Create the D3D Device
	if (FAILED(IDirect3D7_CreateDevice( g_pD3D, &IID_IDirect3DTnLHalDevice, g_pDDSBack, &g_pd3dDevice )))
	{
		// ok, this 1 don't have TnL HAL, we try HAL instead
		if (FAILED(IDirect3D7_CreateDevice( g_pD3D, &IID_IDirect3DHALDevice, g_pDDSBack, &g_pd3dDevice )))
		{
			// Oh no, looks like this PC don't have 3D acceleration
			d3dhardware=FALSE;
			if (FAILED(IDirect3D7_CreateDevice( g_pD3D, &IID_IDirect3DMMXDevice, g_pDDSBack, &g_pd3dDevice )))
			{
				// Oh...even don't have MMX support HEL ?? we better quit !!!!
				D3DFailed( "Direct3D CreateDevice Fail\n",TRUE);
			}
		}
	}
	
	// Then we set the Viewport here
	g_Viewport.dwX=0;
	g_Viewport.dwY=0;
	g_Viewport.dwWidth=SCREEN_WIDTH;
	g_Viewport.dwHeight=SCREEN_HEIGHT;
	g_Viewport.dvMinZ=0.0f;
	g_Viewport.dvMaxZ=1.0f;
	
	IDirect3DDevice7_SetViewport( g_pd3dDevice, &g_Viewport );
	
	g_iBpp=BPP;
	
	// We end the Direct3D intialization
	return TRUE;
}

/*--------------------------------------------------------------------------*/
// D3DShutdown : Direct3D Shutdown
void D3DShutdown(void)
{
	D3DRelease(g_pd3dDevice);
	D3DRelease(g_pD3D);
	D3DRelease(g_pClipper);
	D3DRelease(g_pDDSBack);
	D3DRelease(g_pDDS);
	D3DRelease(g_pDD);
}

/*--------------------------------------------------------------------------*/
// D3DBeginScene : D3D BeginScene call
void D3DBeginScene(void)
{
	IDirect3DDevice7_BeginScene(g_pd3dDevice);
}

/*--------------------------------------------------------------------------*/
// D3DEndScene : D3D EndScene call
void D3DEndScene(void)
{
	IDirect3DDevice7_EndScene(g_pd3dDevice);
}

/*--------------------------------------------------------------------------*/
// D3DClear : Clear a surface area with given color
BOOL D3DClear( D3DCOLOR color )
{
	D3DRECT rcDest;
	
	rcDest.x1=0;
	rcDest.y2=SCREEN_HEIGHT;
	rcDest.y1=0;
	rcDest.x2=SCREEN_WIDTH;

	if (FAILED(IDirect3DDevice7_Clear( g_pd3dDevice, 0, NULL, D3DCLEAR_TARGET, color, 1.0f, 0L )))
	{
		D3DFailed("\nDDClearSurface Fail\n",TRUE);
		return FALSE;
	}
	return TRUE;
}

/*--------------------------------------------------------------------------*/
// D3DFlipWin : Windowed Double buffering flip
BOOL D3DFlipWin(void)
{
	RECT    rcSrc;  // source blit rectangle
	RECT    rcDest; // destination blit rectangle
	POINT   p1, p2;

	GetClientRect(g_hWnd, &rcDest);
	
	p1.x = rcDest.left;
	p1.y = rcDest.top;
	p2.x = rcDest.right;
	p2.y = rcDest.bottom;
	
	ClientToScreen(g_hWnd, &p1);
	ClientToScreen(g_hWnd, &p2);
	
	rcDest.left   = p1.x;
	rcDest.top    = p1.y;
	rcDest.right  = p2.x;
	rcDest.bottom = p2.y;
	
	SetRect(&rcSrc, 0, 0, SCREEN_WIDTH, SCREEN_HEIGHT);
	if (IDirectDrawSurface7_IsLost(g_pDDS) == DDERR_SURFACELOST)
		IDirectDrawSurface7_Restore(g_pDDS);
	if (FAILED(IDirectDrawSurface7_Blt(g_pDDS, &rcDest, g_pDDSBack, &rcSrc, DDBLT_WAIT, NULL)))
	{
		// If cannot Blt, just quit the game
		D3DFailed("D3DFlipWin Failed",TRUE);
		return FALSE;
	}
	
	return TRUE;
}

/*--------------------------------------------------------------------------*/
// TextureSearchCallback : Callback function for enum texture format
HRESULT CALLBACK TextureSearchCallback( DDPIXELFORMAT* pddpf, VOID* param )
{
    if( NULL==pddpf || NULL==param )
        return DDENUMRET_OK;

    TEXTURESEARCHINFO* ptsi = (TEXTURESEARCHINFO*)param;

    // Skip any funky modes
    if( pddpf->dwFlags & (DDPF_LUMINANCE|DDPF_BUMPLUMINANCE|DDPF_BUMPDUDV) )
        return DDENUMRET_OK;

    // Check for palettized formats
    if( ptsi->bUsePalette )
    {
        if( !( pddpf->dwFlags & DDPF_PALETTEINDEXED8 ) )
            return DDENUMRET_OK;

        // Accept the first 8-bit palettized format we get
        memcpy( ptsi->pddpf, pddpf, sizeof(DDPIXELFORMAT) );
        ptsi->bFoundGoodFormat = TRUE;
        return DDENUMRET_CANCEL;
    }

    // Else, skip any paletized formats (all modes under 16bpp)
    if( pddpf->dwRGBBitCount < 16 )
        return DDENUMRET_OK;

    // Skip any FourCC formats
    if( pddpf->dwFourCC != 0 )
        return DDENUMRET_OK;

    // Skip any ARGB 4444 formats (which are best used for pre-authored
    // content designed speciafically for an ARGB 4444 format).
    if( pddpf->dwRGBAlphaBitMask == 0x0000f000 )
        return DDENUMRET_OK;

    // Make sure current alpha format agrees with requested format type
    if( (ptsi->bUseAlpha==TRUE) && !(pddpf->dwFlags&DDPF_ALPHAPIXELS) )
        return DDENUMRET_OK;
    if( (ptsi->bUseAlpha==FALSE) && (pddpf->dwFlags&DDPF_ALPHAPIXELS) )
        return DDENUMRET_OK;

    // Check if we found a good match
    if( pddpf->dwRGBBitCount == ptsi->dwDesiredBPP )
    {
        memcpy( ptsi->pddpf, pddpf, sizeof(DDPIXELFORMAT) );
        ptsi->bFoundGoodFormat = TRUE;
        return DDENUMRET_CANCEL;
    }
    return DDENUMRET_OK;
}

/*--------------------------------------------------------------------------*/
// D3DLoadTexture : Load a BMP to a texture & return it 
LPDIRECTDRAWSURFACE7 D3DLoadTexture( LPCSTR strFilename )
{
	HRESULT hr;
	HBITMAP hbmBitmap;
	HDC hdcBitmap;
    LPDIRECTDRAW7 pDD;
    DDSURFACEDESC2 ddsd;
    D3DDEVICEDESC7 D3DDD;
    LPDIRECTDRAWSURFACE7 pddsSurface=NULL;
    LPDIRECTDRAWSURFACE7 pddsTempSurface=NULL;
    DWORD x, y, i;
    char	tempfile[40];

	sprintf(tempfile,"%s.bmp",strFilename);
    // Load bitmap files
    hbmBitmap = (HBITMAP)LoadImage( NULL, tempfile, IMAGE_BITMAP, 0, 0, LR_LOADFROMFILE|LR_CREATEDIBSECTION );

	BITMAP bm;
    GetObject( hbmBitmap, sizeof(BITMAP), &bm );

	memset(&ddsd,0,sizeof(ddsd));
    ddsd.dwSize = sizeof(ddsd);

	ddsd.dwFlags = DDSD_CAPS | DDSD_HEIGHT |DDSD_WIDTH|DDSD_PIXELFORMAT|DDSD_TEXTURESTAGE;
	ddsd.ddsCaps.dwCaps= DDSCAPS_TEXTURE;
    ddsd.dwWidth = bm.bmWidth;
	ddsd.dwHeight = bm.bmHeight;

	// Turn on texture management for hardware devices
	if (d3dhardware)
		ddsd.ddsCaps.dwCaps2=DDSCAPS2_TEXTUREMANAGE;
	else ddsd.ddsCaps.dwCaps |= DDSCAPS_SYSTEMMEMORY;

	// Adjust width and height to be powers of 2, if the device requires it
    if( D3DDD.dpcTriCaps.dwTextureCaps & D3DPTEXTURECAPS_POW2 )
    {
        for( ddsd.dwWidth=1;  bm.bmWidth>ddsd.dwWidth;   ddsd.dwWidth<<=1 );
        for( ddsd.dwHeight=1; bm.bmHeight>ddsd.dwHeight; ddsd.dwHeight<<=1 );
    }

    // Limit max texture sizes, if the driver can't handle large textures
    DWORD dwMaxWidth  = D3DDD.dwMaxTextureWidth;
    DWORD dwMaxHeight = D3DDD.dwMaxTextureHeight;
    ddsd.dwWidth  = min( ddsd.dwWidth,  ( dwMaxWidth  ? dwMaxWidth  : 256 ) );
    ddsd.dwHeight = min( ddsd.dwHeight, ( dwMaxHeight ? dwMaxHeight : 256 ) );

    // Make the texture square, if the driver requires it
    if( D3DDD.dpcTriCaps.dwTextureCaps & D3DPTEXTURECAPS_SQUAREONLY )
    {
        if( ddsd.dwWidth > ddsd.dwHeight )
        	ddsd.dwHeight = ddsd.dwWidth;
        else	ddsd.dwWidth  = ddsd.dwHeight;
    }

	// Enumerate a correct texture format
    TEXTURESEARCHINFO tsi;
    tsi.bFoundGoodFormat = FALSE;
    tsi.pddpf            = &ddsd.ddpfPixelFormat;
    tsi.dwDesiredBPP     = g_iBpp;
    tsi.bUsePalette      = ( g_iBpp <= 8 );
    tsi.bUseAlpha        = TRUE;
    if( g_iBpp & D3DTEXTR_16BITSPERPIXEL )
        tsi.dwDesiredBPP = 16;
    else if( g_iBpp & D3DTEXTR_32BITSPERPIXEL )
        tsi.dwDesiredBPP = 32;

	if( tsi.bUsePalette )
	{
		if( D3DDD.dpcTriCaps.dwTextureCaps & D3DPTEXTURECAPS_ALPHAPALETTE )
		{
			tsi.bUseAlpha   = TRUE;
			tsi.bUsePalette = TRUE;
		}
		else
		{
			tsi.bUseAlpha   = TRUE;
			tsi.bUsePalette = FALSE;
		}
	}
	g_pd3dDevice->lpVtbl->EnumTextureFormats( g_pd3dDevice, TextureSearchCallback, &tsi );
	// If we couldn't find a format, let's try a default format
    if( FALSE == tsi.bFoundGoodFormat )
    {
        tsi.bUsePalette  = FALSE;
        tsi.dwDesiredBPP = 16;
        g_pd3dDevice->lpVtbl->EnumTextureFormats( g_pd3dDevice, TextureSearchCallback, &tsi );
        // If we still fail, we cannot create this texture
        //if( FALSE == tsi.bFoundGoodFormat )
        //{
        //	D3DFailed("\nCannot found a Alpha format", TRUE);
        //   return NULL;
		//}
    }
    g_pd3dDevice->lpVtbl->GetRenderTarget( g_pd3dDevice,&pddsSurface );
    // Get a DDraw object to create a temporary surface
	IDirectDrawSurface7_GetDDInterface( pddsSurface, (VOID**)&pDD );
    //if (hr!=DD_OK)
	//	D3DFailed("\nLoadImageData GetDDInterface error", TRUE);

    IDirectDraw7_CreateSurface( pDD, &ddsd, &pddsSurface, NULL );

    IDirectDrawSurface7_GetSurfaceDesc( pddsSurface, &ddsd );
    //if (hr!=DD_OK)
    //	D3DFailed("\nLoadImageData GetSurfaceDesc error",TRUE);

    ddsd.dwFlags          = DDSD_CAPS|DDSD_HEIGHT|DDSD_WIDTH|DDSD_PIXELFORMAT|DDSD_TEXTURESTAGE;
    ddsd.ddsCaps.dwCaps   = DDSCAPS_TEXTURE|DDSCAPS_SYSTEMMEMORY;
    ddsd.ddsCaps.dwCaps2  = 0L;
    ddsd.dwWidth          = bm.bmWidth;
    ddsd.dwHeight         = bm.bmHeight;

    if( FAILED( IDirectDraw7_CreateSurface( pDD, &ddsd, &pddsTempSurface, NULL ) ) )
    {
        pDD->lpVtbl->Release(pDD);
        pDD=NULL;
        return NULL;
    }
	// Get a DC for the bitmap
	hdcBitmap = CreateCompatibleDC( NULL );
	if( NULL == hdcBitmap )
	{
		IDirectDrawSurface7_Release( pddsTempSurface );
		pddsTempSurface=NULL;
		pDD->lpVtbl->Release(pDD);
        pDD=NULL;
		return NULL;
	}
    SelectObject( hdcBitmap, hbmBitmap );

	// Handle 8bit palettized textures. Need to attach a palette
    if( ddsd.ddpfPixelFormat.dwRGBBitCount == 8 )
    {
        LPDIRECTDRAWPALETTE  pPalette;
        DWORD dwPaletteFlags = DDPCAPS_8BIT|DDPCAPS_ALLOW256;
        DWORD pe[256];
        WORD  wNumColors = GetDIBColorTable( hdcBitmap, 0, 256, (RGBQUAD*)pe );

        // Create the color table
        for( i=0; i<wNumColors; i++ )
        {
            pe[i] = RGB( GetBValue(pe[i]), GetGValue(pe[i]), GetRValue(pe[i]) );
			if( pe[i] != 0x00000000 )
				pe[i] |= 0xff000000;
        }
        // Add DDPCAPS_ALPHA flag for textures with transparent pixels
		dwPaletteFlags |= DDPCAPS_ALPHA;

        // Create & attach a palette
        IDirectDraw7_CreatePalette( pDD, dwPaletteFlags, (PALETTEENTRY*)pe, &pPalette, NULL );
        IDirectDrawSurface7_SetPalette( pddsTempSurface, pPalette );
        IDirectDrawSurface7_SetPalette( pddsSurface, pPalette );
        IDirectDrawPalette_Release( pPalette );
        pPalette=NULL;
    }

    // Copy the bitmap image to the surface.
    HDC hdcSurface;
    if( SUCCEEDED( IDirectDrawSurface7_GetDC( pddsTempSurface, &hdcSurface ) ) )
    {
        BitBlt( hdcSurface, 0, 0, bm.bmWidth, bm.bmHeight, hdcBitmap, 0, 0, SRCCOPY );
        IDirectDrawSurface7_ReleaseDC( pddsTempSurface, hdcSurface );
    }
    DeleteDC( hdcBitmap );

    // Copy the temp surface to the real texture surface
    IDirectDrawSurface7_Blt( pddsSurface, NULL, pddsTempSurface, NULL, DDBLT_WAIT, NULL );
    // Done with the temp surface
    IDirectDrawSurface7_Release( pddsTempSurface );

    // For textures with real alpha (not palettized), set transparent bits
	if( ddsd.ddpfPixelFormat.dwRGBAlphaBitMask )
	{
		// Lock the texture surface
		//DDSURFACEDESC2 ddsd;
		ddsd.dwSize = sizeof(ddsd);
		while( IDirectDrawSurface7_Lock( pddsSurface, NULL, &ddsd, 0, NULL ) ==  DDERR_WASSTILLDRAWING );

		DWORD dwAlphaMask = ddsd.ddpfPixelFormat.dwRGBAlphaBitMask;
		DWORD dwRGBMask   = ( ddsd.ddpfPixelFormat.dwRBitMask | ddsd.ddpfPixelFormat.dwGBitMask |
                              ddsd.ddpfPixelFormat.dwBBitMask );
		// Colorkey on black, yes BLACK 
		DWORD dwColorkey  = 0x00000000;

		// Add an opaque alpha value to each non-colorkeyed pixel
		for( y=0; y<ddsd.dwHeight; y++ )
		{
			WORD*  p16 = (WORD*)((BYTE*)ddsd.lpSurface + y*ddsd.lPitch);
			DWORD* p32 = (DWORD*)((BYTE*)ddsd.lpSurface + y*ddsd.lPitch);

			for( x=0; x<ddsd.dwWidth; x++ )
			{
				if( ddsd.ddpfPixelFormat.dwRGBBitCount == 16 )
				{
					if( ( *p16 &= dwRGBMask ) != dwColorkey )
						*p16 |= dwAlphaMask;
					p16++;
				}
				if( ddsd.ddpfPixelFormat.dwRGBBitCount == 32 )
				{
					if( ( *p32 &= dwRGBMask ) != dwColorkey )
						*p32 |= dwAlphaMask;
					p32++;
				}
			}
		}
		IDirectDrawSurface7_Unlock( pddsSurface, NULL );
    }
    IDirectDraw7_Release(pDD);

	return pddsSurface;
}







#endif //_D3D7WRAP_H_